// the arrow objects

#include "arrow.h"
#include "level.h"
#include "rocket.h"
#include "particle.h"



// global list of all arrows
ThingList<Arrow> arrows;



Arrow::Arrow(double x, double y, double vx, double vy)
{
    mPos.set(x,y);
    mVel.set(vx,vy);
    mRadius = 2;
	mBounces = 3;
}



// arrow update function
bool Arrow::update()
{
	Vector oldpos = mPos;

	mPos += mVel;

    vector<LevelCollision> collisions;

	if (check_level_collision_line (oldpos, mPos, collisions)) {
		mPos += collisions[0].normal * collisions[0].dist;
		mVel -= collisions[0].normal * 2 * (mVel | collisions[0].normal);
		
		trigger_particles(TRIGGER_ARROW_WALL, mPos, -collisions[0].normal);

		if (!--mBounces) return false;

		if (mBounces == 1)
			mVel /= 2;
	}

	if (mBounces == 1)
		mVel.y += 0.025;

	ThingList<Rocket>::iterator it;
	for (it = rockets.begin(); it != rockets.end(); it++) {
		Rocket& rocket = *it;

		// reject anything to the left of us
		if (rocket.mPos.x + rocket.mRadius < mPos.x - mRadius)
			continue;

		// if it's to the right, reject all the rest
		if (rocket.mPos.x - rocket.mRadius > mPos.x + mRadius)
			break;

		if (rocket.mDead) continue;

        float distsq = (rocket.mPos - mPos).square_magnitude();

        if(distsq < (rocket.mRadius + mRadius) * (rocket.mRadius + mRadius) ) {
			trigger_particles(TRIGGER_ARROW_ROCKET, mPos, rocket.mVel);
			rocket.mDead = true;
			return false;
		}
	}

	return true;
}



// arrow render function
void Arrow::render(BITMAP *bmp)
{
	Vector front = ~mVel * mRadius * 1;

    Vector right(-front.y, front.x);

    Vector p[3];	
    
    p[0] = mPos + front + scroll;
    p[1] = mPos + right + scroll;
    p[2] = mPos - right + scroll;

    int col = makecol(255, 128, 0);

    triangle(bmp, p[0].x, p[0].y, p[1].x, p[1].y, p[2].x, p[2].y, col);

	p[0] = mPos + scroll;
	p[1] = mPos + scroll - front*3;

	line(bmp, p[0].x, p[0].y, p[1].x, p[1].y, col);

	p[2] = p[1] - front*2 - right;

	line(bmp, p[1].x, p[1].y, p[2].x, p[2].y, col);

	p[2] = p[1] - front*2 + right;

	line(bmp, p[1].x, p[1].y, p[2].x, p[2].y, col);
}

