#include "editor.h"

#include <stdio.h>



static int prop_proc (void)
{
	static char actual_name[128];
	static char name[128];

	DIALOG d[] = {
    /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key)   (flags)  (d1)        (d2)      (dp)                 (dp2) (dp3) */
	{  d_shadow_box_proc, 0,    0,  480,  100,     0,    0,     0,   0,         0,          0,      0,                      0,    0 },
	{  d_button_proc,    44,   70,   64,   16,     0,    0,    13,   D_EXIT,    0,          0,      "OK",                   0,    0 },
	{  d_button_proc,   132,   70,   64,   16,     0,    0,    27,   D_EXIT,    0,          0,      "Cancel",               0,    0 },
	{  d_ctext_proc,    240,   10,    0,    0,     0,    0,     0,   0,         0,          0,      "Level properties",     0,    0 },
	{  d_rtext_proc,    100,   30,    0,    0,     0,    0,     0,   0,         0,          0,      "Name:",                0,    0 },
	{  d_edit_proc,     120,   30,  240,   10,     0,    0,     0,   0,       127,          0,      name,                   0,    0 },
	{0}
	};
	
	if (levelN.mName != actual_name) {
		strcpy (actual_name, levelN.mName);
	}

	strcpy (name, actual_name);

	set_dialog_color (d, 15, 0);
	centre_dialog (d);

	if (popup_dialog (d, 4) == 1) {
		strcpy (actual_name, name);
		levelN.mName = actual_name;
	}

	return D_O_K;
}


static int clear_proc (void)
{
	int a = alert3 ("Save level before clearing?", NULL, NULL, "Yes", "No", "Cancel", 'y', 'n', 27);
	if (a == 1) save();
	if (a != 3) {
		// TODO: meemory leaks!
		vertices.clear();
		edges.clear();
		objects.clear();
	}
	return D_O_K;
}


static int load_proc (void)
{
	int a = alert3 ("Save level before reverting?", NULL, NULL, "Yes", "No", "Cancel", 'y', 'n', 27);
	if (a == 1) save();
	if (a != 3) {
		load();
	}
	return D_O_K;
}


static int save_proc (void)
{
	save();
	alert ("Saved", NULL, NULL, "OK", NULL, 13, 27);
	return D_O_K;
}


static int quit_proc (void)
{
	int a = alert3 ("Save level before quitting?", NULL, NULL, "Yes", "No", "Cancel", 'y', 'n', 27);
	if (a == 1) save();
	if (a != 3) quit = true;
	return D_O_K;
}



static MENU file_menu[] = {
	/*  text           proc        child  flags  dp  */
	{	"&Properties", prop_proc,  NULL,  0,     0 },
	{   "&Clear",      clear_proc, NULL,  0,     0 },
	{   "&Revert",     load_proc,  NULL,  0,     0 },
	{   "&Save",       save_proc,  NULL,  0,     0 },
	{   "&Quit",       quit_proc,  NULL,  0,     0 },
	{0}
};


extern DIALOG dialog[];
static int mode_add_proc (void) { dialog[2].proc (MSG_CHAR, dialog+2, '1'); return D_O_K; }
static int mode_edit_proc (void) { dialog[2].proc (MSG_CHAR, dialog+2, '2'); return D_O_K; }
static int mode_obj_proc (void) { dialog[2].proc (MSG_CHAR, dialog+2, '3'); return D_O_K; }

static MENU mode_menu[] = {
	/*  text                proc             child  flags  dp  */
	{   "Add points\t&1",   mode_add_proc,   NULL,  0,     0 },
	{   "Edit points\t&2",  mode_edit_proc,  NULL,  0,     0 },
	{	"Edit objects\t&3", mode_obj_proc,   NULL,  0,     0 },
	{0}
};


static char *mode_strings[] = {
	"add points",
	"edit points",
	"edit objects"
};
	


static MENU menu_bar[] = {
	/*  text        proc    child       flags  dp  */
	{   "&File",    NULL,   file_menu,  0,     0   },
	{   "&Mode",    NULL,   mode_menu,  0,     0   },
	{0}
};


static int leveledit_proc (int msg, DIALOG *d, int c);


DIALOG dialog[] = {
    /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key) (flags)  (d1)                    (d2)  (dp)              (dp2) (dp3) */
	{  d_clear_proc,      0,    0,    0,    0,     0,    0,     0,      0,    0,                      0,   0,                  0,    0 },
	{  d_menu_proc,       0,    0,    0,    0,     0,    0,     0,      0,    0,                      0,   menu_bar,           0,    0 },
	{  leveledit_proc,    8,   16,  624,  456,     0,    0,     0,      0,    0,                      0,   0,                  0,    0 },
	{ 0 }
};


static char edit_object_id[32];
static char edit_object_target[4][32];
static char edit_object_data[4][32];


static const char *object_type_lister (int index, int *list_size)
{
	if (index < 0) {
		*list_size = OBJTYPE_COUNT;
		return NULL;
	}

	return objnames[index];
}


static DIALOG dialog_edit_object[] = {
    /* (dialog proc)     (x)   (y)   (w)   (h)   (fg)  (bg)  (key)   (flags)  (d1)        (d2)      (dp)                 (dp2) (dp3) */
	{  d_shadow_box_proc, 0,    0,  480,  200,     0,    0,     0,        0,    0,          0,      0,                      0,    0 },
	{  d_button_proc,    44,  170,   64,   16,     0,    0,    13,   D_EXIT,    0,          0,      "OK",                   0,    0 },
	{  d_button_proc,   132,  170,   64,   16,     0,    0,    27,   D_EXIT,    0,          0,      "Cancel",               0,    0 },
	{  d_ctext_proc,    240,   10,    0,    0,     0,    0,     0,        0,    0,          0,      "Edit object",          0,    0 },
	{  d_rtext_proc,    100,   30,    0,    0,     0,    0,     0,        0,    0,          0,      "Id:",                  0,    0 },
	{  d_rtext_proc,    100,   45,    0,    0,     0,    0,     0,        0,    0,          0,      "Type:",                0,    0 },
	{  d_rtext_proc,    100,  100,    0,    0,     0,    0,     0,        0,    0,          0,      "Targets:",             0,    0 },
	{  d_text_proc,     120,   30,    0,    0,     0,    0,     0,        0,    0,          0,      edit_object_id,         0,    0 },
	{  d_list_proc,     120,   45,   80,   45,     0,    0,     0,        0,    0,          0,      object_type_lister,     0,    0 },
	{  d_edit_proc,     120,  100,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_target[0],  0,    0 },
	{  d_edit_proc,     120,  115,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_target[1],  0,    0 },
	{  d_edit_proc,     120,  130,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_target[2],  0,    0 },
	{  d_edit_proc,     120,  145,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_target[3],  0,    0 },
	{  d_rtext_proc,    340,  100,    0,    0,     0,    0,     0,        0,    0,          0,      "Extra data:",          0,    0 },
	{  d_edit_proc,     360,  100,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_data[0],    0,    0 },
	{  d_edit_proc,     360,  115,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_data[1],    0,    0 },
	{  d_edit_proc,     360,  130,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_data[2],    0,    0 },
	{  d_edit_proc,     360,  145,   80,   10,     0,    0,     0,        0,    8,          0,      edit_object_data[3],    0,    0 },
	{ 0 }
};



static DIALOG_PLAYER *player;


void gui_init (void)
{
	dialog[0].w = SCREEN_W;
	dialog[0].h = SCREEN_H;
	set_dialog_color (dialog, 15, 0);
	player = init_dialog (dialog, 2);
}

void gui_update (void)
{
	update_dialog (player);
}



static int leveledit_proc (int msg, DIALOG *d, int c)
{
	static float xscroll = 0, yscroll = 0;
	static float scale = 0.35;
	static int mode = 1;
	static Vtx *prev_vtx = NULL;
	static BITMAP *bmp;
	
	switch (msg) {
		case MSG_WANTFOCUS: return D_WANTFOCUS;

		case MSG_START: {
			bmp = create_bitmap (SCREEN_W, SCREEN_H);
		}

		case MSG_DRAW: {
			clear(bmp);
			rect (bmp, d->x, d->y, d->x+d->w, d->y+d->h, d->fg);
			set_clip (bmp, d->x+1, d->y+1, d->x+d->w-1, d->y+d->h-1);

			{
				list<Edge *>::iterator it;
				for (it = edges.begin(); it != edges.end(); it++) {
					line (bmp, ((*it)->v1->x - xscroll) * scale, ((*it)->v1->y - yscroll) * scale, ((*it)->v2->x - xscroll) * scale, ((*it)->v2->y - yscroll) * scale, d->fg);
					float nx = (*it)->v2->y - (*it)->v1->y;
					float ny = (*it)->v1->x - (*it)->v2->x;
					float dist = nx*nx+ny*ny;
					if (dist > 0.01) {
						nx /= sqrt(dist);
						ny /= sqrt(dist);
						int x = (int)((((*it)->v1->x + (*it)->v2->x) / 2 - xscroll) * scale);
						int y = (int)((((*it)->v1->y + (*it)->v2->y) / 2 - yscroll) * scale);
						line (bmp, x, y, x+nx*4, y+ny*4, 8);
					}
				}
			}

			{
				list<Vtx *>::iterator it;
				for (it = vertices.begin(); it != vertices.end(); it++)
					circle (bmp, (int)(((*it)->x - xscroll) * scale), (int)(((*it)->y - yscroll) * scale), 4, (*it == prev_vtx) ? 1 : d->fg);
			}

			{
				list<Obj *>::iterator it;
				list<Obj *>::iterator it2;
				for (it = objects.begin(); it != objects.end(); it++) {
					int c = 1;
					float r = 20;
					switch ((*it)->type) {
						case OBJTYPE_NONE: c = 8; r = 20; break;
						case OBJTYPE_SHIP: c = 2; r = 20; break;
						case OBJTYPE_ENEMY: c = 3; r = 30; break;
					}
					circle (bmp, (int)(((*it)->pos->x - xscroll) * scale), (int)(((*it)->pos->y - yscroll) * scale), (int)(r * scale), c);

					if ((*it)->pos == prev_vtx) {
						for (int i = 0; i < 4; i++)
							if ((*it)->targets[i] >= 0) {
								for (it2 = objects.begin(); it2 != objects.end(); it2++) {
									if ((*it2)->id == (*it)->targets[i])
										line (bmp,
											((*it)->pos->x - xscroll) * scale, ((*it)->pos->y - yscroll) * scale,
											((*it2)->pos->x - xscroll) * scale, ((*it2)->pos->y - yscroll) * scale,
											1);
								}
							}
					}
				}
			}

			set_clip (bmp, 0, 0, SCREEN_W-1, SCREEN_H-1);

			blit (bmp, screen, d->x, d->y, d->x, d->y, d->w+1, d->h+1);

			text_mode (mode+8);
			textprintf_right (screen, font, d->x+d->w-2, d->y+2, d->fg, "%s (%d)", mode_strings[mode-1], mode);

			text_mode (d->bg);
			textprintf_right (screen, font, d->x+d->w-2, d->y+10, d->fg, "Zoom: % 3d%%", (int)(scale*100+0.5));

			return D_O_K;
		}

		case MSG_CLICK: {
			if (mouse_b & 4) {

				/* Middle drag to scroll */
				int dx, dy;
				get_mouse_mickeys (&dx, &dy);
				while (mouse_b & 4) {
					get_mouse_mickeys (&dx, &dy);
					if (dx || dy) {
						xscroll -= (float)dx / scale;
						yscroll -= (float)dy / scale;
						scare_mouse();
						d->proc (MSG_DRAW, d, 0);
						unscare_mouse();
					}
					vsync();
				}

			} else if (mouse_b & 1) {

				/* Left click function depends on mode */
				switch (mode) {
					case 1: {
						/* Add points */
						int x = (int)(mouse_x / scale + xscroll + 0.5);
						int y = (int)(mouse_y / scale + yscroll + 0.5);
						Vtx *v = find_or_add_vertex (x, y, (int)(4/scale + 0.5));
						if (prev_vtx && v != prev_vtx)
							edges.push_back (new Edge (prev_vtx, v));
						prev_vtx = v;
						scare_mouse();
						d->proc (MSG_DRAW, d, 0);
						unscare_mouse();
						break;
					}
					case 2: {
						/* Edit points */
						int x = (int)(mouse_x / scale + xscroll + 0.5);
						int y = (int)(mouse_y / scale + yscroll + 0.5);
						Vtx *v = find_vertex (x, y, (int)(4/scale + 0.5));
						prev_vtx = v;
						if (v) {
							while (mouse_b & 1) {
								int newx = (int)(mouse_x / scale + xscroll + 0.5);
								int newy = (int)(mouse_y / scale + yscroll + 0.5);
								if (x != newx || y != newy) {
									x = v->x = newx;
									y = v->y = newy;
									scare_mouse();
									d->proc (MSG_DRAW, d, 0);
									unscare_mouse();
								}
								vsync();
							}
						}
						scare_mouse();
						d->proc (MSG_DRAW, d, 0);
						unscare_mouse();
						break;
					}
					case 3: {
						/* Edit objects */
						int x = (int)(mouse_x / scale + xscroll + 0.5);
						int y = (int)(mouse_y / scale + yscroll + 0.5);
						Vtx *v = find_vertex (x, y, (int)(4/scale + 0.5));
						prev_vtx = v;
						if (v) {
							Obj *obj = NULL;

							/* Find an object, if one exists at this vertex */
							list<Obj *>::iterator it;
							for (it = objects.begin(); it != objects.end(); it++)
								if ((*it)->pos == v) break;

							if (it != objects.end()) {
								obj = *it;
							} else {
								/* See if the user wants to create a new object */
								if (alert ("Create object?", NULL, NULL, "Yes", "No", 'y', 'n') == 1) {
									obj = new Obj (v, OBJTYPE_NONE);
									objects.push_back (obj);
								}
							}

							/* If we found or created an object, edit it */
							if (obj) {

								sprintf (edit_object_id, "%d", obj->id);

								for (int i = 0; i < 4; i++)
									sprintf (edit_object_target[i], "%d", obj->targets[i]);

								for (int i = 0; i < 4; i++)
									sprintf (edit_object_data[i], "%d", obj->data[i]);

								dialog_edit_object[8].d1 = obj->type;

								set_dialog_color (dialog_edit_object, 15, 0);
								centre_dialog (dialog_edit_object);

								if (popup_dialog (dialog_edit_object, 0) == 1) {

									for (int i = 0; i < 4; i++)
										obj->targets[i] = atoi (edit_object_target[i]);

									for (int i = 0; i < 4; i++)
										obj->data[i] = atoi (edit_object_data[i]);

									obj->type = dialog_edit_object[8].d1;

									if (obj->type == OBJTYPE_NONE) {
										if (alert ("Delete object (type = NONE)?", NULL, NULL, "Yes", "No", 'y', 'n') == 1) {
											for (it = objects.begin(); it != objects.end(); it++)
												if (*it == obj) {
													objects.erase (it);
													break;
												}
											delete obj;
										}
									}

								}

								scare_mouse();
								d->proc (MSG_DRAW, d, 0);
								unscare_mouse();
							}
						}
						break;
					}
				}

			} else if (mouse_b & 2) {

				/* Right click tends to delete things */
				switch (mode) {
					case 1:
					case 2: {
						int x = (int)(mouse_x / scale + xscroll + 0.5);
						int y = (int)(mouse_y / scale + yscroll + 0.5);
						Vtx *v = find_vertex (x, y, (int)(4/scale + 0.5));
						if (prev_vtx == v) prev_vtx = NULL;

						if (v) {

							delete_vertex (v);
							scare_mouse();
							d->proc (MSG_DRAW, d, 0);
							unscare_mouse();

						} else {

							Edge *e = find_edge (x, y, (int)(4/scale + 0.5));
							if (e) {
								delete_edge (e);
								scare_mouse();
								d->proc (MSG_DRAW, d, 0);
								unscare_mouse();
							}

						}
					}
				}
			}

			return D_O_K;
		}

		case MSG_CHAR: {
			int ret = D_O_K;

			float oldscale = scale;
			if ((c & 0xff) == '+') {
				scale *= 1.3;
				ret = D_USED_CHAR;
			} else if ((c & 0xff) == '-') {
				scale /= 1.3;
				ret = D_USED_CHAR;
			}
			if (scale != oldscale) {
				xscroll += d->w/2 / oldscale;
				yscroll += d->h/2 / oldscale;
				xscroll -= d->w/2 / scale;
				yscroll -= d->h/2 / scale;
			}

			int oldmode = mode;
			switch (c & 0xff) {
				case '1': mode = 1; ret = D_USED_CHAR; break;
				case '2': mode = 2; ret = D_USED_CHAR; break;
				case '3': mode = 3; ret = D_USED_CHAR; break;
			}
			if (mode != oldmode) {
				prev_vtx = NULL;
			}

			scare_mouse();
			d->proc (MSG_DRAW, d, 0);
			unscare_mouse();

			return ret;
		}

		case MSG_WHEEL: {
			while (c < 0) { d->proc (MSG_CHAR, d, '-'); c++; }
			while (c > 0) { d->proc (MSG_CHAR, d, '+'); c--; }
			return D_O_K;
		}

		default:
			return d_clear_proc (msg, d, c);
	}
}

