#include "editor.h"

#include <stdio.h>
#include <string.h>
#include <ctype.h>


void save (void)
{
	/* get level number */
	const char *ch = strrchr (levelN.mSourceName, '.') - 1;
	while (isdigit(*ch)) ch--;
	int level_number = atoi (ch+1);

	FILE *fp = fopen (levelN.mSourceName, "w");
	if (!fp) {
		alert ("Error opening file", levelN.mSourceName, NULL, "OK", NULL, 13, 27);
		return;
	}

	fprintf (fp, "// level #%d\n\n", level_number);

	fprintf (fp, "#include \"level.h\"\n");
	fprintf (fp, "#include \"objects.h\"\n\n\n");

	fprintf (fp, "static char sourcename[] = __FILE__;\n\n");
	fprintf (fp, "static char name[] = \"%s\";\n\n", levelN.mName); // TODO: quote weird characters?

	fprintf (fp, "#ifndef EDITOR\n");
	fprintf (fp, "#define levelN level%d\n", level_number);
	fprintf (fp, "#endif\n\n\n");



	fprintf (fp, "static int points[] =\n{");

	for (list<Edge *>::iterator it = edges.begin(); it != edges.end(); it++)
		fprintf (fp, "\n\t%d, %d,\n\t%d, %d,\n", (*it)->v1->x, (*it)->v1->y, (*it)->v2->x, (*it)->v2->y);

	fprintf (fp, "};\n\n\n\n");



	fprintf (fp, "static ObjData objs[] = {\n");

	for (list<Obj *>::iterator it = objects.begin(); it != objects.end(); it++)
		fprintf (fp, "\t{ %d, %d, OBJTYPE_%s, %d, { %d, %d, %d, %d }, { %d, %d, %d, %d } },\n",
			(*it)->pos->x, (*it)->pos->y, objnames[(*it)->type], (*it)->id,
			(*it)->targets[0], (*it)->targets[1], (*it)->targets[2], (*it)->targets[3],
			(*it)->data[0], (*it)->data[1], (*it)->data[2], (*it)->data[3]
		);

	/* hack: if there are no objects, the output won't compile so you can't run the editor again to add objects */
	if (objects.size() == 0) fprintf (fp, "\t{0}\n");

	fprintf (fp, "};\n\n\n\n");



	fprintf (fp, "Level levelN =\n");
	fprintf (fp, "{\n");
	fprintf (fp, "\tname,\n");
	fprintf (fp, "\tsourcename,\n");
	fprintf (fp, "\tARRAY_SIZE(points)/4,\n");
	fprintf (fp, "\tpoints,\n");
	fprintf (fp, "\tARRAY_SIZE(objs),\n");
	fprintf (fp, "\tobjs\n");
	fprintf (fp, "};\n");

	fclose (fp);
}

