#ifndef __gf_included_editor_h
#define __gf_included_editor_h


#include "main.h"

#include "level.h"


struct Vtx {
	int x, y;
	Vtx(int _x, int _y) : x(_x), y(_y) {}
};
struct Edge {
	Vtx *v1, *v2;
	Edge(Vtx *_v1, Vtx *_v2) : v1(_v1), v2(_v2) {}
};
struct Obj {
	static int next_id;
	Vtx *pos;
	int type, id;
	int targets[4];
	int data[4];

	Obj (Vtx *_pos, int _type) : id(next_id++), type(_type), pos(_pos)
		{
			for (int i = 0; i < 4; i++) targets[i] = -1;
			for (int i = 0; i < 4; i++) data[i] = 0;
		}

	Obj (Vtx *_pos, int _type, int _id, int *_targets, int *_data)
		: id(_id), type(_type), pos(_pos)
		{
			for (int i = 0; i < 4; i++) targets[i] = _targets[i];
			for (int i = 0; i < 4; i++) data[i] = _data[i];
			if (next_id <= id) next_id = id+1;
		}
};

extern list<Vtx *> vertices;
extern list<Edge *> edges;
extern list<Obj *> objects;


extern Level levelN;


/* "Load" and save */
void load (void);
void save (void);


/* GUI */
void gui_init (void);
void gui_update (void);


/* Time to go home? */
extern bool quit;


Vtx *find_or_add_vertex (int x, int y, int tolerance = 1);
Vtx *find_vertex (int x, int y, int tolerance = 1);
Vtx *delete_vertex (int x, int y, int tolerance = 1);
Vtx *delete_vertex (Vtx *vtx);

Edge *find_edge (float x, float y, int tolerance = 1);
Edge *delete_edge (Edge *edge);


#endif

