// the enemy objects

#include <stdlib.h>
#include <stdio.h>

#include "enemy.h"
#include "bitmaps.h"
#include "level.h"
#include "rocket.h"
#include "lighting.h"
#include "panel.h"
#include "particle.h"
#include "gameplay.h"
#include "people.h"

// global list of all enemies
ThingList<Enemy> enemies;



Enemy::Enemy(double x, double y, int type, int id, int targets[], float rocket_gravity)
    : Thing(), mRocketGravity(rocket_gravity)
{
    mPos.set(x,y);

    mBmp = *(people[type].bmp);
    mName = people[type].name;

    // ever wished you'd used an array?

    mTL.set( mPos.x - mBmp->w/2, mPos.y - mBmp->h);
    mBR.set( mPos.x + mBmp->w/2, mPos.y);

    mHealth = 10;
    mOrigHealth = 10;

    mHealthDisplayPercent = 1;

    mID = id;

    for(int i=0; i<4; i++) {
        if(targets[i] == -1)
            continue;

        mTargets.push_back(targets[i]);
    }
}



// enemy update function
bool Enemy::update()
{
    if(mFireThisFrame) {

        // choose someone to shoot at

        int e = rand() % mTargets.size();

        Enemy* enemy=NULL;

        for(int i=0; i<4; i++) {
        
            ThingList<Enemy>::iterator it = enemies.begin();
            while(it != enemies.end()) {
                enemy = &(*it);
                
                if(enemy->mID == mTargets[e])
                    goto got_an_enemy;
                
                
                it++;
            }
            
            e = (e+1) % mTargets.size();
            
        }

		enemy = NULL;

    got_an_enemy:

        if(enemy) {

            // George: do your aiming here!
			Vector startpos = mPos - Vector(0, mBmp->h);
			Vector target = enemy->mPos - Vector(0, enemy->mBmp->h/2) - startpos;
			target.y = -target.y;

			float g = mRocketGravity * gravity_multiplier;

			float theta = atan ((target.y + target.magnitude()) / target.x);
			if (theta < 0) theta += M_PI;
			float v = sqrt (-0.5 * g * target.x * target.x * (1 + tan(theta) * tan(theta)) / (target.y - target.x * tan(theta)));
			
			Vector dir (cos(theta), -sin(theta));

            rockets.add(Rocket(mID, startpos, dir, dir * v, g));

			trigger_particles(TRIGGER_FIRE_ROCKET, startpos, dir*v);
        }
    }


    float newpercent = (float)mHealth / mOrigHealth;

    mHealthDisplayPercent = newpercent * 0.2 + mHealthDisplayPercent * 0.8;

	if ((int)(mHealthDisplayPercent*100) < (int)(mPanel->mHealth*100))
		mPanel->mHitTime = 70*2;

	mPanel->mHealth = mHealthDisplayPercent;

	return ((int)(mHealthDisplayPercent*100) > 0);
}



// enemy render function
void Enemy::render(BITMAP *bmp)
{
    draw_lit_dude(bmp, mBmp, (int)(mPos.x - mBmp->w/2 + scroll.x), (int)(mPos.y-mBmp->h + scroll.y));

	/*
    rect(bmp, mPos.x - mBmp->w/2 + scroll.x, mPos.y-mBmp->h + scroll.y, 
         mPos.x + mBmp->w/2 + scroll.x, mPos.y + scroll.y,
         makecol(128,0,0));
	*/

    char buf[50];
    sprintf(buf,"%d", (int)(mHealthDisplayPercent * 100.0));

    textout_centre(bmp, font, buf, (int)(mPos.x + scroll.x), (int)(mTL.y + scroll.y - 10), (mPanel->mHitTime) ? makecol(255,0,0) : makecol(128, 128, 128));
}

