// menu system

#include "main.h"
#include "frontend.h"
#include "game.h"
#include "lighting.h"
#include "level.h"
#include "heart_trails.h"
#include "particle.h"
#include "highscores.h"
#include "people.h"
#include "sound.h"



static int sel = 0;

static int num_levels;


static int drawing_person;
static int next_person;
static int person_timer;
static enum { ENTERING, LEAVING } person_state;
static float person_x;
static float person_vel;


void init_frontend()
{
	in_frontend = true;
	
	num_levels = 0;
	
	while (level_table[num_levels]) {
		num_levels++;
    }

    drawing_person = -1;
    person_timer = 70*5;
    next_person = 0;

	fade_out(16);
	clear(screen);
	set_palette(our_palette);
    
	do {
	} while (key[KEY_UP] || key[KEY_DOWN] || key[KEY_SPACE] || key[KEY_ENTER] || key[KEY_ESC]);
	
	clear_keybuf();
}



bool update_frontend()
{
	while (keypressed()) {
	
		switch (readkey() >> 8) {
		
			case KEY_UP:
				if (sel > 0) {
					sel--;
					menu_click(false);
				}
				break;
			
			case KEY_DOWN:
				if (sel < num_levels+3) {
					sel++;
					menu_click(false);
				}
				break;
			
			case KEY_SPACE:
			case KEY_ENTER:
				if (sel == num_levels+3) {
					return false;
				}
                else if (sel == num_levels+2) {
					menu_click(true);
                    use_particles = (use_particles + 1) % 3;
                }
                else if (sel == num_levels+1) {
					menu_click(true);
                    heart_trails = !heart_trails;
                }
				else if (sel == num_levels) {
					menu_click(true);

					if (use_bumpmap)
						use_lights = use_bumpmap = false;
					else if (use_lights)
						use_bumpmap = true;
					else
						use_lights = true;
				}
				else {
					menu_click(true);
					level = level_table[sel];
					init_game();
				}
				break;
			
			case KEY_ESC:
				return false;
		}
	}

    person_timer--;
    if(person_timer <= 0) {
        if(drawing_person == -1) {
            // need to choose someone to draw
            drawing_person = next_person;
            next_person = (next_person + 1) % num_people;
            player_whoosh();

            person_timer = 70 * 3;
            person_state = ENTERING;
            person_x = 640;
            person_vel = 0;
        }
        else if(person_state == ENTERING) {
            person_state = LEAVING;
            person_timer = 70 * 3;
            player_whoosh();
        }
        else {
            drawing_person=-1;
        }

    }

    if(drawing_person != -1) {
        float wanted_x;

        switch(person_state) {
        case ENTERING:
            wanted_x = SCREEN_W * 2.0 / 3.0;
            break;

        case LEAVING:
        default:
            wanted_x = SCREEN_W+100;
            break;
        }

        float wanted_vel = (wanted_x - person_x) * 0.1;

        person_vel = wanted_vel * 0.1 + person_vel * 0.9;
        person_x += person_vel;
    }

	return true;
}



void render_frontend(BITMAP *bmp)
{
	int sel_col = (retrace_count & 16) ? makecol(160, 160, 0) : makecol(255, 255, 0);
	int desel_col = makecol(255, 255, 255);

	clear_to_color(bmp, makecol(32, 70, 96));

	if(drawing_person != -1) {
        BITMAP* pb = *(people[drawing_person].bmp);
        draw_sprite(bmp, pb, person_x, SCREEN_H - pb->h - 17);
        textout_centre(bmp, font, people[drawing_person].name, (int)person_x + pb->w/2 + 1, SCREEN_H - 16 , makecol(0,0,0));
        textout_centre(bmp, font, people[drawing_person].name, (int)person_x + pb->w/2, SCREEN_H - 16, makecol(255,255,255));
    }


	text_mode(-1);
	
	textout_centre(bmp, font, "By Doctor Pepper, Sir George of Foot, and Shawn Hargreaves", SCREEN_W/2+1, 65, 0);
	textout_centre(bmp, font, "By Doctor Pepper, Sir George of Foot, and Shawn Hargreaves", SCREEN_W/2,   64, makecol(255, 255, 255));

	int y = 160;
	int dy = 24;
	
	for (int i=0; i<num_levels; i++) {
		textprintf(bmp, font, 50+1, y+i*dy+1, 0,                                "Level #%d - %s", i+1, level_table[i]->mName);
		textprintf(bmp, font, 50,   y+i*dy, (sel == i) ? sel_col : desel_col, "Level #%d - %s", i+1, level_table[i]->mName);

		textprintf_right(bmp, font, SCREEN_W-50+1, y+i*dy+1, 0,                                "%10s %10d", level_table[i]->mHighscoreName, level_table[i]->mHighscore);
		textprintf_right(bmp, font, SCREEN_W-50, y+i*dy, (sel == i) ? sel_col : desel_col, "%10s %10d", level_table[i]->mHighscoreName, level_table[i]->mHighscore);
	}

    y += 16;

	textout(bmp, font, (use_bumpmap) ?        "Lighting : bumpmapped" : (use_lights) ? "Lighting : simple" : "Lighting : none", 50+1, y+num_levels*dy+1, 0);
	textout(bmp, font, (use_bumpmap) ?        "Lighting : bumpmapped" : (use_lights) ? "Lighting : simple" : "Lighting : none", 50,   y+num_levels*dy, (sel == num_levels) ? sel_col : desel_col);

    textout(bmp, font, (heart_trails) ?       "Trails   : on" : "Trails   : off", 50+1, y+(num_levels+1)*dy+1, 0);
    textout(bmp, font, (heart_trails) ?       "Trails   : on" : "Trails   : off", 50,   y+(num_levels+1)*dy, (sel == num_levels+1) ? sel_col : desel_col);

    textout(bmp, font, (use_particles == 0) ? "Particles: off" : (use_particles == 1) ? "Particles: non-colliding" : "Particles: colliding", 50+1, y+(num_levels+2)*dy+1, 0);
    textout(bmp, font, (use_particles == 0) ? "Particles: off" : (use_particles == 1) ? "Particles: non-colliding" : "Particles: colliding", 50,   y+(num_levels+2)*dy, (sel == num_levels+2) ? sel_col : desel_col);

    y += 16;

	textout(bmp, font, "Quit", 50+1, y+(num_levels+3)*dy+1, 0);
	textout(bmp, font, "Quit", 50,   y+(num_levels+3)*dy, (sel == num_levels+3) ? sel_col : desel_col);
}


