#include "main.h"
#include "game.h"
#include "level.h"
#include "heart.h"
#include "sound.h"

#include <stdio.h>

void load_highscores()
{
    int i=0;

    while(level_table[i]) {
        const char* highscorename;
        
        char buf[80];
        sprintf(buf,"%s_name", level_table[i]->mSourceName);

        highscorename = get_config_string("Highscores", buf, "Nobody");

        strcpy(level_table[i]->mHighscoreName, highscorename);

        sprintf(buf,"%s_score", level_table[i]->mSourceName);

        level_table[i]->mHighscore = get_config_int("Highscores", buf, 0);

        i++;
    }

}


void save_highscores()
{
    int i=0;
	while (level_table[i]) {
        char buf[80];
        sprintf(buf,"%s_name", level_table[i]->mSourceName);

        set_config_string("Highscores", buf, level_table[i]->mHighscoreName);

        sprintf(buf,"%s_score", level_table[i]->mSourceName);

        set_config_int("Highscores", buf, level_table[i]->mHighscore);

        i++;
    }
}


extern BITMAP* double_buffer;

static char name[10];
static char* pos;

void render_highscore_screen(BITMAP* bmp)
{
    int black = makecol(0,0,0);
    int white = makecol(255,255,255);

    clear_to_color(bmp, black);
    
    textout_centre(bmp, font, "Congratulations!", 320, 50, white);
    textout_centre(bmp, font, "You have a new high score!", 320, 70, white);

    textout_centre(bmp, font, "Please enter your name", 320, 200-16, white);

    char buf[50];
    sprintf(buf, "%s_", name);
    textout_centre(bmp, font, buf, 320, 200+16, white);
}

void highscore_screen()
{
    clear_keybuf();

    name[0] = 0;
    pos = name;

    while(1) {
        render_highscore_screen(double_buffer);

        blit(double_buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);

        int key = readkey();

        switch( key>>8 ) {
        case KEY_ESC:
        case KEY_ENTER:
			menu_click(true);
            return;

        case KEY_BACKSPACE:
			menu_click(false);

            if(pos != name)
                pos--;

            *pos = 0;

            continue;
        }

        if(pos-name < ARRAY_SIZE(name)-1) {
			menu_click(false);
            *pos = key;
            pos++;
            *pos = 0;
        }

    }

}

void do_highscores()
{
    if(heart->mScore > level->mHighscore) {
        highscore_screen();

        level->mHighscore = heart->mScore;

        if(strlen(name) > 0)
            strncpy(level->mHighscoreName, name, 10);
        else
            sprintf(level->mHighscoreName, "Mr Nobody");
    }
}


