// the main program loop

#include "main.h"
#include "frontend.h"
#include "game.h"
#include "bitmaps.h"
#include "lighting.h"
#include "plasma.h"
#include "heart_trails.h"
#include "particle.h"
#include "sound.h"
#include "level.h"
#include "highscores.h"

// double buffer
BITMAP *double_buffer;



// are we in the frontend, or the game?
bool in_frontend = true;



// nice set of colors
PALETTE our_palette;



// sets up a good palette
static void create_palette()
{
	int i;

	// 64 greyscale values
	for (i=0; i<64; i++)
		our_palette[i].r = our_palette[i].g = our_palette[i].b = i;

	// next 180 colors hold a rainbow spectrum
	for (i=0; i<180; i++) {
		our_palette[i+64].r = ((i%6)       + 1) * 63/6;
		our_palette[i+64].g = (((i/6)%6)   + 1) * 63/6;
		our_palette[i+64].b = (((i/6/6)%5) + 1) * 63/5;
	}

	// final 12 are unused
	for (i=64+180; i<256; i++)
		our_palette[i].r = our_palette[i].g = our_palette[i].b = 0;

	set_palette(our_palette);
}



// per-frame update
static bool do_update()
{
	if (in_frontend)
		return update_frontend();
	else
		return update_game();
}



// per-frame render
static void render_frame()
{
	if (in_frontend)
		render_frontend(double_buffer);
	else
		render_game(double_buffer);

	blit(double_buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}



// not at all sure what this comment is for
int main(int argc, char *argv[])
{
	allegro_init();
	install_keyboard();
	install_timer();
	
	set_color_depth(8);
	set_gfx_mode(GFX_SAFE, 640, 480, 0, 0);

	init_sound();

	create_palette();

	double_buffer = create_bitmap(SCREEN_W, SCREEN_H);
	
    init_bitmaps();
	init_plasma();
	init_lighting();
	init_particles();

	set_config_file("cupid.cfg");
	
	use_lights  = (get_config_int(NULL, "lights",  use_lights ) != 0);
	use_bumpmap = (get_config_int(NULL, "bumpmap", use_bumpmap) != 0);
	heart_trails = (get_config_int(NULL, "trails", heart_trails) != 0);
	use_particles = get_config_int(NULL, "particles", use_particles);

    load_highscores();

	init_frontend();
	
	int last_time = retrace_count;

	for (;;) {
		do {
		} while (last_time == retrace_count);

		if (!do_update())
			break;

		last_time++;

		if (last_time == retrace_count)
			render_frame();
	}
	
	fade_out(16);

	destroy_bitmap(double_buffer);

	set_config_int(NULL, "lights", use_lights);
	set_config_int(NULL, "bumpmap", use_bumpmap);
    set_config_int(NULL, "trails", heart_trails);
    set_config_int(NULL, "particles", use_particles);

    save_highscores();


	return 0;
}

END_OF_MAIN()

