// HUD display routines

#include "panel.h"
#include "heart.h"
#include "bitmaps.h"
#include "gameplay.h"

// list of player names/health etc
list<PlayerPanel> player_panel;



// right align position
static int right;



static Vector gameover_pos;
static Vector gameover_vel;
static int gameover_countdown;

// panel init function
void init_panel()
{
	player_panel.clear();
	
	right = -1;

    gameover_pos.set(-bmp_gameover->w, 200 - bmp_gameover->h * 0.5);
    gameover_vel.set(0,0);
    gameover_countdown = 70;
}



// per-frame panel update
void update_panel()
{
	for (list<PlayerPanel>::iterator it = player_panel.begin(); it != player_panel.end(); it++) {
		if (it->mHitTime)
			it->mHitTime--;
	}
}




// draws the panel text
void render_panel(BITMAP *bmp)
{
	text_mode(-1);
	
	int std_col  = makecol(192, 192, 192);
	int hit_col  = makecol(255, 0,   0  );
	int dead_col = makecol(48,  48,  48 );
	
	list<PlayerPanel>::iterator it;

	// score
	textprintf(bmp, font, 16, 16, makecol(255, 128, 0), "Score: %d", heart->mScore);

	// measure name widths first time we're run
	if (right < 0) {
		for (it = player_panel.begin(); it != player_panel.end(); it++) {
			int r = 16 + strlen(it->mName)*8 + 8*5;
			right = MAX(right, r);
		}
	}
	

	// draw healths 
	int i = player_panel.size();

	for (it = player_panel.begin(); it != player_panel.end(); it++) {
		int col;

		if (it->mHitTime)
			col = hit_col;		
		else if ((int)(it->mHealth*100) <= 0)
			col = dead_col;
		else
			col = std_col;

		textprintf(bmp, font, 16, SCREEN_H-8-i*16, col, "%s:", it->mName);

		textprintf_right(bmp, font, right,   SCREEN_H-8-i*16, col, "%d", (int)(it->mHealth*100));

		i--;
	}


    if(game_over) {
        gameover_countdown--;

        if(gameover_countdown <= 0) {
            Vector wanted_pos(320 - bmp_gameover->w/2, 200 - bmp_gameover->h/2);
            
            Vector wanted_vel = (wanted_pos - gameover_pos) * 0.2;
            
            if(wanted_vel.square_magnitude() > 0.001) {
                gameover_vel = wanted_vel * 0.1 + gameover_vel * 0.9;
                gameover_pos += gameover_vel;
            }
            else
                gameover_pos = wanted_pos;
            
            draw_sprite(bmp, bmp_gameover, gameover_pos.x, gameover_pos.y);
        }
    }
}

