// the particle objects

#ifndef __PARTICLE_H__
#define __PARTICLE_H__


#include "thinglist.h"



// enable/disable particles
extern int use_particles;



// the particle class
class Particle : public Thing
{
public:

	Particle (const Vector &pos, const Vector &vel, int lifespan, float gravity, int type, bool collide, class Flower *flower = NULL);

	// update and render handlers
	bool update();
	void render(BITMAP *bmp);

	Vector mOldPos;
	Vector mVel;
	float mGravity;
	int mLifeSpan;
	int mType;
	bool mCollide;

	class Flower *mFlower;

	enum Type {
		TYPE_THRUST,
		TYPE_SPARK,
		TYPE_FLOWER,
	};
};



// global list of all particles
extern ThingList<Particle> particles;



// generate graphics etc
void init_particles (void);



// trigger function
enum TRIGGER_TYPE
{
	TRIGGER_PLAYER_WALL,
	TRIGGER_PLAYER_ROCKET,
	TRIGGER_FIRE_ARROW,
	TRIGGER_ARROW_WALL,
	TRIGGER_ARROW_ROCKET,
	TRIGGER_FIRE_ROCKET,
	TRIGGER_ROCKET_WALL,
	TRIGGER_ROCKET_PERSON,
	TRIGGER_KILLED_PERSON,
    TRIGGER_ROCKET_SILENTDEATH  // for when the rockets self-destruct on end of game
};

void trigger_particles(TRIGGER_TYPE type, const Vector &pos, const Vector &vel);




#endif		// __PARTICLE_H__


