// the rocket objects

#include "rocket.h"
#include "level.h"
#include "enemy.h"
#include "particle.h"
#include "gameplay.h"


// global list of all rockets
ThingList<Rocket> rockets;

static BITMAP *rocket_bmp = NULL;



Rocket::Rocket(int owner, const Vector& start, const Vector& dir, const Vector& vel, float gravity)
    : mOwnerID(owner), mFront(dir), mVel(vel), mGravity(gravity)
{
	int i;

    mPos = start;
    mRadius = 5;

	if (!rocket_bmp) {
		rocket_bmp = create_bitmap((int)(mRadius*2), (int)(mRadius*4));
		
		clear(rocket_bmp);

		for (i=(int)mRadius; i>0; i--)
			circlefill(rocket_bmp, mRadius, mRadius, i, makecol((int)(255-i*128/mRadius), 0, 0));

		for (i=(int)mRadius; i<rocket_bmp->h; i++) {
			int c = (int)(255 * (rocket_bmp->h - i) / (rocket_bmp->h - mRadius));

			hline(rocket_bmp, mRadius*2/3, i, mRadius*4/3, makecol(c, 0, 0));
		}
	}
}



// rocket update function
bool Rocket::update()
{
    Vector oldpos = mPos;

    // gravity
    mVel += Vector(0, mGravity);
        

    mPos += mVel;


    vector<LevelCollision> collisions;

    if(check_level_collision_line(oldpos, mPos, collisions)) {
		trigger_particles(TRIGGER_ROCKET_WALL, mPos, mVel - collisions[0].normal * (mVel | collisions[0].normal) * 1.5);
        mDead = true;
        return false;
    }

    if(game_over) {
        trigger_particles(TRIGGER_ROCKET_SILENTDEATH, mPos, mVel * 2);
        mDead = true;
        return false;
    }

    // check for collision with enemies
    // remember that the enemy list is sorted by x coord
    ThingList<Enemy>::iterator it;

    it = enemies.begin();

    // skip all enemies off to our left
    while(it != enemies.end()) {
        Enemy& enemy = *it;

        if(enemy.mBR.x >= mPos.x - mRadius)
            break;

        it++;
    }

    // only bother looking closely at enemies that aren't off to the right
    while(it != enemies.end()) {
        Enemy& enemy = *it;

        if(enemy.mTL.x > mPos.x + mRadius)
            break;

        // test against the enemy's bounding box

        if(mPos.x > enemy.mTL.x && mPos.x < enemy.mBR.x &&
           mPos.y > enemy.mTL.y && mPos.y < enemy.mBR.y) {

            if(enemy.mID != mOwnerID) {
                // COLLISION!!!
                if(!game_over)
                    enemy.mHealth--;
                
                if (enemy.mHealth <= 0) {
                    enemy.mHealth = 0;
					trigger_particles(TRIGGER_KILLED_PERSON, mPos, mVel);
				}
				else
					trigger_particles(TRIGGER_ROCKET_PERSON, mPos, mVel);

                return false;
            }
        }

        it++;
    }

	return true;
}



// rocket render function
void Rocket::render(BITMAP *bmp)
{
	// if old Allegro version: 
	#define fixatan2 fatan2
	pivot_sprite(bmp, rocket_bmp, (int)(mPos.x+scroll.x), (int)(mPos.y+scroll.y), (int)mRadius, (int)mRadius, fixatan2(ftofix(mVel.y), ftofix(mVel.x)) + itofix(64));
}

