// template class for managing lists of objects

#ifndef __THINGLIST_H__
#define __THINGLIST_H__


#include "thing.h"



// holds a list of game objects
template<class T>
class ThingList
{
public:
	
	// adds a new item to the list
	void add(const T &t)
	{
		mList.push_back(t);
		mList.sort();
	}


	// updates all the entries in the list, removing dead ones
	void update()
	{
		list<T>::iterator it = mList.begin();
		
		while (it != mList.end()) {
			if ((it->mDead) || (!it->update()))
				it = mList.erase(it);
			else
				it++;
		}

		mList.sort();
	}
	
	
	// renders all the entries in the list
	void render(BITMAP *bmp)
	{
		// todo: trivial reject those which are offscreen

		for (list<T>::iterator it = mList.begin(); it != mList.end(); it++)
			it->render(bmp);
	}


	// clears the entire contents of the list
	void clear()
	{
		mList.clear();
	}
	

	// iterator interface
	typedef list<T>::iterator iterator;

	iterator begin() { return mList.begin(); }
	iterator end()   { return mList.end();   }

	T &front() { return mList.front(); }
	T &back()  { return mList.back();  }

	int size() { return mList.size(); }
	

private:
	// the worker STL list
	list<T> mList;
};




#endif		// __THINGLIST_H__


